﻿using System;
using System.Linq;
using System.Web;
using System.Web.Http;
using System.Web.Mvc;
using System.Web.Optimization;
using System.Web.Routing;
using System.IO;
using FanvilDeviceManageServer.Filters;
using System.Net;
using System.Threading;
using FanvilDeviceManageServer.Modules.Mail;
using FanvilDeviceManageServer.Modules.ProxyServer;
using FanvilDeviceManageServer.Modules.Rule;
using FanvilDeviceManageServer.Service;
using FanvilDeviceManageServer.Modules.I18n;
using FanvilDeviceManageServer.Core;
using FanvilDeviceManageServer.Maintain.Performance;
using FanvilDeviceManageServer.Modules.Rpc;

namespace FanvilDeviceManageServer
{
    public class MvcApplication : HttpApplication
    {
        protected void Application_Start()
        {
            AreaRegistration.RegisterAllAreas();

            WebApiConfig.Register(GlobalConfiguration.Configuration);
            FilterConfig.RegisterGlobalFilters(GlobalFilters.Filters);
            RouteConfig.RegisterRoutes(RouteTable.Routes);
            BundleConfig.RegisterBundles(BundleTable.Bundles);
            AuthConfig.RegisterAuth();

            MongoDBService mongoService = new MongoDBService();
            if (mongoService.IsServiceEnable())
            {
                Manage.GetInstance().ForceStart();
                OnlineChecker.GetInstance().ForceStart();
                UdpServer.GetInstance()?.ForceStart();
                MailService.GetInstance().ForceStart();
                RuleServer.GetInstance().ForceStart();
                NotifyService.GetInstance().ForceStart();
                ProxyController.GetInstance().ForceStart();
                PerformCollector.GetInstance().Initialize();
            }

            new RpcListener().Start();
            Translator.GetInstance().SetupFileWatcher();
            //PipeManager.GetInstance("FDMS_SERVER").Initialize();
        }

        protected void Application_Error(object sender, EventArgs e)
        {
            Exception ex = Server.GetLastError();
            string lang = Thread.CurrentThread.CurrentUICulture.Name;
            if (lang == null) lang = "zh-CN";

            if (ex is HttpAntiForgeryException)
            {
                Response.Clear();
                Server.ClearError();
                Response.Redirect("/" + lang + "/Account/Login");
            }
            else if (ex is HttpException)
            {
                Response.Clear();
                Server.ClearError();
                Response.Redirect("/" + lang + "/Home/Index");
            }
            else
            {
                string logFilePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "systemLogs.txt");
                using (StreamWriter sw = new StreamWriter(logFilePath, true, System.Text.Encoding.UTF8))
                {
                    sw.Write(HttpContext.Current.Error);
                }
            }
        }

        protected void Application_EndRequest(object sender, EventArgs e)
        {
            HttpResponse response  = HttpContext.Current.Response;
            if (response.Headers.AllKeys.Contains(HttpAuthorizeAttribute.RESP_HEADER_NAME) && response.StatusCode == (int)HttpStatusCode.Redirect)
            {
                response.StatusCode = (int)HttpStatusCode.Unauthorized;
                response.StatusDescription = HttpAuthorizeAttribute.UNAUTHORIZED;
            }
        }
    }
}