﻿if (typeof Object.create !== "function") {
    Object.create = function (obj) {
        function F() { }
        F.prototype = obj;
        return new F();
    };
}

(function ($, window, document, undefined) {
    "use strict";

    var DeviceHandler = {
        Init: function (options) {
            this.PrepareOptions(options, $.deviceHandler.options);
            this.Process();
        },
        PrepareOptions: function (options, options_to_extend) {
            var opts = {};
            if ((typeof options === "string") || (options instanceof Array)) {
                opts.text = options;
            } else {
                opts = options;
            }
            this._options = $.extend({}, options_to_extend, opts);
        },
        Process: function () {
            if (this._options.container == null) {
                return;
            }
            this.Setup();
        },
        Setup: function () {
            this.Bind();
        },
        Bind: function () {
            if (this._options.features != null && this._options.features.length > 0) {
                var featureBtns = $(this._options.container).find("button");
                var features = this.Features();
                var utils = this.Utils();

                for (var i = 0; i < featureBtns.length; i++) {
                    var toggleName = $(featureBtns[i]).data("toggle");

                    if (toggleName == null || toggleName.length == 0) continue;

                    //$(featureBtns[i]).attr("disabled", "disabled");

                    for (var j = 0; j < this._options.features.length; j++) {
                        if (toggleName == this._options.features[j].toggleName) {
                            var callback = utils.GetToggleCallBack(toggleName);

                            if (callback != null) {
                                $(featureBtns[i]).click(features[this._options.features[j].functionName]);
                                //$(featureBtns[i]).removeAttr("disabled");
                            }
                            break;
                        }
                    }
                }

                var manageChkbox = utils.GetManageCheckBox();

                $(manageChkbox).click(function () {
                    var chked = $(this).is(':checked');
                    utils.CheckAll(chked);
                });
            }
        },
        Utils: function () {
            var parent = this;
            var utils = {
                CheckAll: function (flag) {
                    var chkboxs = this.GetAllCheckBox();
                    for (var i = 0; i < chkboxs.length; i++) {
                        chkboxs[i].checked = flag;
                    }
                },
                GetManageCheckBox: function () {
                    var chkboxs = this.GetAllCheckBox();
                    for (var i = 0; i < chkboxs.length; i++) {
                        var role = $(chkboxs[i]).data("role");

                        if (role == "manage") {
                            return chkboxs[i];
                        }
                    }
                    return null;
                },
                GetAllCheckBox: function () {
                    var chkboxs = $(parent._options.container).find("input[type=checkbox]");
                    return chkboxs;
                },
                GetCheckedIds: function () {
                    var ids = new Array();
                    var chkboxs = this.GetAllCheckBox();
                    for (var i = 0; i < chkboxs.length; i++) {
                        var role = $(chkboxs[i]).data("role");

                        if (role != "manage") {
                            if (chkboxs[i].checked) {
                                ids[ids.length] = chkboxs[i].id;
                            }
                        }
                    }
                    return ids;
                },
                GetCheckedMacs: function () {
                    var macs = new Array();
                    var chkboxs = this.GetAllCheckBox();
                    for (var i = 0; i < chkboxs.length; i++) {
                        var role = $(chkboxs[i]).data("role");

                        if (role != "manage") {
                            if (chkboxs[i].checked) {
                                macs[macs.length] = $(chkboxs[i]).data("mac");
                            }
                        }
                    }
                    return macs;
                },
                GetToggleCallBack: function (toggleName) {
                    for (var i = 0; i < parent._options.callbacks.length; i++) {
                        if (parent._options.callbacks[i].toggleName == toggleName) {
                            return parent._options.callbacks[i].callback;
                        }
                    }
                    return null;
                }
            };

            return utils;
        },
        Features: function () {
            var parent = this, utils = parent.Utils();

            var ReturnIds = function (callback) {
                if (callback != null) {
                    var devIds = utils.GetCheckedIds();

                    if (devIds.length > 0) {
                        callback(devIds);
                    } else {
                        callback(null);
                    }
                }

                utils.CheckAll(false);
            },
            ReturnMacs = function (callback) {
                if (callback != null) {
                    var devMacs = utils.GetCheckedMacs();

                    if (devMacs.length > 0) {
                        callback(devMacs);
                    } else {
                        callback(null);
                    }
                }

                utils.CheckAll(false);
            };

            var features = {
                Reboot: function () {
                    var callback = utils.GetToggleCallBack("reboot");

                    if (callback != null) {
                        var devMacs = utils.GetCheckedMacs();

                        if (devMacs.length > 0) {
                            var acsReqs = new Array(), reboot = { Method: "Reboot", Params: null };

                            for (var i = 0; i < devMacs.length; i++) {
                                var methods = new Array();
                                methods[methods.length] = reboot;
                                acsReqs[acsReqs.length] = { Mac: devMacs[i], AcsMethods: methods };
                            }

                            callback(JSON.stringify({ AcsRequestBeanList: acsReqs }));
                        } else {
                            callback(null);
                        }
                    }

                    utils.CheckAll(false);
                },
                Delete: function () {
                    var callback = utils.GetToggleCallBack("delete");
                    ReturnIds(callback);
                },
                CheckOnline: function () {
                    var callback = utils.GetToggleCallBack("checkonline");
                    ReturnIds(callback);
                },
                Upgrade: function () {
                    var callback = utils.GetToggleCallBack("upgrade");
                    ReturnIds(callback);
                },
                Provision: function () {
                    var callback = utils.GetToggleCallBack("provision");
                    ReturnIds(callback);
                },
                Parameter: function () {
                    var callback = utils.GetToggleCallBack("parameter");
                    ReturnIds(callback);
                },
                ParameterTest: function () {
                    var callback = utils.GetToggleCallBack("parametertest");
                    ReturnIds(callback);
                },
                Reprovision: function () {
                    var callback = utils.GetToggleCallBack("reprovision");
                    ReturnMacs(callback);
                },
                CameraPreview: function () {
                    var callback = utils.GetToggleCallBack("camera");
                    ReturnIds(callback);
                },
                OpenDoor: function () {
                    var callback = utils.GetToggleCallBack("opendoor");
                    ReturnIds(callback);
                },
                ManageCard: function () {
                    var callback = utils.GetToggleCallBack("managecard");
                    ReturnIds(callback);
                },
                ManageEgsLog: function () {
                    var callback = utils.GetToggleCallBack("egslog");
                    ReturnIds(callback);
                },
                SetIp: function () {
                    var callback = utils.GetToggleCallBack("setip");
                    ReturnIds(callback);
                },
                FactoryReset: function () {
                    var callback = utils.GetToggleCallBack("reset");

                    if (callback != null) {
                        var devMacs = utils.GetCheckedMacs();

                        if (devMacs.length > 0) {
                            var acsReqs = new Array(), reset = { Method: "FactoryReset", Params: null };

                            for (var i = 0; i < devMacs.length; i++) {
                                var methods = new Array();
                                methods[methods.length] = reset;
                                acsReqs[acsReqs.length] = { Mac: devMacs[i], AcsMethods: methods };
                            }

                            callback(JSON.stringify({ AcsRequestBeanList: acsReqs }));
                        } else {
                            callback(null);
                        }
                    }

                    utils.CheckAll(false);
                }
            };
            
            return features;
        }
    };

    $.deviceHandler = function (options) {
        var deviceHandler = Object.create(DeviceHandler);
        deviceHandler.Init(options);

        return { };
    };

    $.deviceHandler.options = {
        container: null,
        features: [
            {
                toggleName: "reboot",
                functionName: "Reboot"
            },
            {
                toggleName: "delete",
                functionName: "Delete"
            },
            {
                toggleName: "checkonline",
                functionName: "CheckOnline"
            },
            {
                toggleName: "upgrade",
                functionName: "Upgrade"
            },
            {
                toggleName: "provision",
                functionName: "Provision"
            },
            {
                toggleName: "parameter",
                functionName: "Parameter"
            },
            {
                toggleName: "parametertest",
                functionName: "ParameterTest"
            },
            {
                toggleName: "reprovision",
                functionName: "Reprovision"
            },
            {
                toggleName: "camera",
                functionName: "CameraPreview"
            },
            {
                toggleName: "opendoor",
                functionName: "OpenDoor"
            },
            {
                toggleName: "managecard",
                functionName: "ManageCard"
            },
            {
                toggleName: "egslog",
                functionName: "ManageEgsLog"
            },
            {
                toggleName: "setip",
                functionName: "SetIp"
            },
            {
                toggleName: "reset",
                functionName: "FactoryReset"
            }

        ],
        callbacks: [
            {
                toggleName: "reboot",
                callback: null
            },
            {
                toggleName: "delete",
                callback: null
            },
            {
                toggleName: "checkonline",
                callback: null
            },
            {
                toggleName: "upgrade",
                callback: null
            },
            {
                toggleName: "provision",
                callback: null
            },
            {
                toggleName: "parameter",
                callback: null
            },
            {
                toggleName: "reprovision",
                callback: null
            },
            {
                toggleName: "camera",
                callback: null
            },
            {
                toggleName: "opendoor",
                callback: null
            },
            {
                toggleName: "managecard",
                callback: null
            },
            {
                toggleName: "egslog",
                callback: null
            },
            {
                toggleName: "setip",
                callback: null
            },
            {
                toggleName: "reset",
                callback: null
            }
        ]
    };
})(jQuery, window, document);