﻿if (typeof Object.create !== "function") {
    Object.create = function (obj) {
        function F() { }
        F.prototype = obj;
        return new F();
    };
}

(function ($, window, document, undefined) {
    "use strict";

    var ProcessHandler = {
        Init: function (options) {
            this.PrepareOptions(options, $.processHandler.options);
            this.Process();
        },
        PrepareOptions: function (options, options_to_extend) {
            var opts = {};
            if ((typeof options === "string") || (options instanceof Array)) {
                opts.text = options;
            } else {
                opts = options;
            }
            this._options = $.extend({}, options_to_extend, opts);
        },
        Process: function () {
            if (this._options.data == null || this._options.statusList == null || this._options.statusList.length == 0) {
                return;
            }
            this.Setup();
        },
        Setup: function () {
            this.Bind();
        },
        SseDo: function (processView) {
            var Processing = function () {
                var chdViews = $(processView.view).children();
                if (chdViews != null && chdViews.length > 0) {
                    for (var i = 0; i < chdViews.length; i++) {
                        $(chdViews[i]).addClass("hidden");
                    }
                }
                var showView = "<span class='icon-spin icon-spinner show-view' style='color: green;'></span>";
                $(processView.view).append(showView);
            },
                Fail = function () {
                    Recover();
                    var tr = GetParent("TR");
                    if (tr != null) {
                        $(tr).css("background-color", "red");
                        $(tr).css("color", "white");
                    }
                },
                Done = function () {
                    Recover();
                    var tr = GetParent("TR");
                    if (tr != null) {
                        $(tr).css("background-color", "green");
                        $(tr).css("color", "white");
                    }
                },
                Recover = function () {
                    var chdViews = $(processView.view).children();
                    if (chdViews != null && chdViews.length > 0) {
                        for (var i = 0; i < chdViews.length; i++) {
                            $(chdViews[i]).removeClass("hidden");
                            if ($(chdViews[i]).hasClass("show-view")) $(chdViews[i]).addClass("hidden");
                        }
                    }
                },
                GetParent = function (tag) {
                    var parent = null;
                    var isTr = false;

                    try {
                        parent = $(processView.view).parent();
                        var MAX_TRY = 10;
                        var tryCount = 0;
                        while (!isTr && tryCount < MAX_TRY) {
                            tryCount++;
                            if ($(parent).is(tag)) {
                                isTr = true;
                                break;
                            }
                            else parent = $(parent).parent();
                        }
                        if (!isTr) parent = null;
                    } catch (err) { parent = null; }
                    return parent;
                };

            var lang = this._options.lang;
            if (lang == null) lang = "zh-CN";

            Processing();

            var sseClient = new SseClient("/" + lang + "/Sse/GetProcessStatus?Key=" + processView.status.Key);
            sseClient.OnMessage = function (data) {
                if (data == null || data.trim().length == 0) return;
                var resp = JSON.parse(data);

                if (resp == null || sseClient.Count >= 100) {
                    Fail();
                    sseClient.Stop();
                } else if (resp.Completed) {
                    Done();
                    sseClient.Stop();
                }
            };
            sseClient.OnError = function (err) {
                console.log("Error Message: " + err);
            };
        },
        Play: function () {
            if (this._processViewList == null || this._processViewList.length == 0) return;

            for (var i = 0; i < this._processViewList.length; i++) {
                this.SseDo(this._processViewList[i]);
            }
        },
        Bind: function () {
            var processItemList = $("." + this._options.viewClass);
            var viewDataList = new Array();

            if (processItemList != null && processItemList.length > 0) {
                for (var i = 0; i < this._options.statusList.length; i++) {
                    var status = this._options.statusList[i];
                    for (var j = 0; j < processItemList.length; j++) {
                        var mac = $(processItemList[j]).data(this._options.data);
                        if (status.Mac == mac) {
                            var viewData = {
                                status: status,
                                view: processItemList[j]
                            };
                            viewDataList[viewDataList.length] = viewData;
                            break;
                        }
                    }
                }
            }

            if (viewDataList.length > 0) {
                this._processViewList = viewDataList;
                this.Play();
            }
        }
    };

    $.processHandler = function (options) {
        var processHandler = Object.create(ProcessHandler);
        processHandler.Init(options);

        return { };
    };

    $.processHandler.options = {
        data: null,
        viewClass: "process-item",
        lang: null,
        statusList: null
    };
})(jQuery, window, document);