﻿if (typeof Object.create !== "function") {
    Object.create = function (obj) {
        function F() { }
        F.prototype = obj;
        return new F();
    };
}

(function ($, window, document, undefined) {
    "use strict";

    var StatusBar = {
        init: function (options) {
            this.prepareOptions(options, $.statusBar.options);
            this.process();
        },
        prepareOptions: function (options, options_to_extend) {
            var _options = {};
            if ((typeof options === "string") || (options instanceof Array)) {
                _options.text = options;
            } else {
                _options = options;
            }

            this.ProcessQueueList = new Array();
            this.isRunning = false;
            this.isDone = false;
            this.processIdx = 0;
            this.options = $.extend({}, options_to_extend, _options);
        },
        process: function () {
            if (this.options.container == null || this.options.statusNames == null) {
                return;
            }
            this.setup();
        },
        setup: function () {
            var _statusBarContent = "";

            this._statusBarEle = this._statusBarEle || $("<div></div>", { class: "status-bar position-relative clearfix" });

            _statusBarContent += "<span class='bar-text' style='left: -17px;'>" + this.options.statusNames[0] + "</span><span class='status-ball'></span>";

            var len = this.options.statusNames.length;

            
            for (var i = 1; i < len; i++) {
                var offLeft = i * 134 - 27 - i*4;

                _statusBarContent += "<span class='status-pipe position-relative'><i class='progress-show progress-show-item'></i></span><span class='bar-text' style='left: " + offLeft + "px;'>" + this.options.statusNames[i] + "</span>";
                _statusBarContent += "<span class='status-ball'></span>";
            }
            
            this._statusBarEle.html(_statusBarContent);
            this.options.container.html(this._statusBarEle);          
            this.currentIdx = 0;
        },
        animate: function (obj, color) {
            if (color == null || color.trim().length == 0) color = "red";      
            $(obj).css({ "background": color, "color": color });

        },
        moveBar: function (obj, width, time) {
            var mover = $(obj).find("i.progress-show");          

            mover.animate({ "width": "+=" + width + "" }, time);
        },
        processPipe: function (isNext) {
            if (this.processIdx + 1 > this.options.statusNames.length) return;
           
            var pipe = null, time = 100;

            if (this.isDone) time = 50;

            if (isNext) {
                pipe = this._statusBarEle.find(".status-pipe")[this.processIdx - 1];
                this.moveBar(pipe, "25px", time);
            } else {
                pipe = this._statusBarEle.find(".status-pipe")[this.processIdx];               
                this.moveBar(pipe, "75px", time*3);
            }
        },
        getCurrentStatus: function () {
            return this.currentIdx - 1;
        },
        logicCtrl: function (color) {         
            var isFirst = false, isLast = null;
            if (color == "red") {
                isLast = true;
            } else {
                isLast = false;
            }

            if (this.currentIdx == 0) isFirst = true;
            else if (this.currentIdx + 1 > this.options.statusNames.length) return;
            else if (this.currentIdx + 1 == this.options.statusNames.length) isLast = true;
            
            var ball = this._statusBarEle.find(".status-ball")[this.currentIdx];
            var text = this._statusBarEle.find(".bar-text")[this.currentIdx];          
           
            var targetObjs = new Array();
            var ballObj = {
                target: ball,
                color: color
            };
            var textObj = {
                target: text,
                color: color
            };
            targetObjs[targetObjs.length] = ballObj;
            targetObjs[targetObjs.length] = textObj;

            var processJson = {
                isFirst: isFirst,
                isLast: isLast,
                targetObjs: targetObjs
            };

            this.ProcessQueueList[this.ProcessQueueList.length] = processJson;

            var _self = this;
            
            var process = function () {
                if (_self.processIdx < _self.ProcessQueueList.length) {
                    var procData = _self.ProcessQueueList[_self.processIdx];
                    _self.processAnimation(procData, process);
                } else {
                    _self.isRunning = false;
                }
            };

            if (!this.isRunning) {
                this.isRunning = true;
                process();
            }
            this.currentIdx++;
        },
        processAnimation: function (processJson, callback) {
            if (processJson == null) {
                if (callback != null) callback();
                return;
            }

            var _self = this;

            var procPipe = function () {
                if (!processJson.isLast) {
                    _self.processPipe(false);
                    var time = 300;
                    if (_self.isDone) time = 150;
                    window.setTimeout(procDone, time);
                } else {
                    procDone();
                }
            },
            procBall = function () {
                var targetObjs = processJson.targetObjs;

                if (targetObjs != null && targetObjs.length > 0) {
                    for (var i = 0; i < targetObjs.length; i++) {
                        _self.animate(targetObjs[i].target, targetObjs[i].color);
                    }
                }

                window.setTimeout(procPipe, 10);
            },
            procDone = function () {
                _self.processIdx++;
                if (callback != null) callback();
            };

            if (!processJson.isFirst) {
                this.processPipe(true);
                var time = 100;
                if (_self.isDone) time = 50;
                window.setTimeout(procBall, time);
            } else {
                procBall();
            }
        },
        done: function () {
            this.isDone = true;
            var count = this.options.statusNames.length - this.ProcessQueueList.length;          
            for (var i = 0; i < count; i++) {
                this.next();
            }
        },
        failed: function () {
            this.logicCtrl("red");
           
        },
        next: function () {
            this.logicCtrl("#4688f1");
        }
    };

    $.statusBar = function (options) {
        var statusBar = Object.create(StatusBar);
        statusBar.init(options);

        return {
            next: function () {
                statusBar.next();
            },
            done: function () {
                statusBar.done();
            },
            failed: function () {
                statusBar.failed();
            },
            getCurrentStatus: function () {
                return statusBar.getCurrentStatus();
            }
        };
    };

    $.statusBar.options = {
        container: null,
        statusNames: null
    };
})(jQuery, window, document);