"use strict"
var XmlParser = function () {
    var self = this;
    
    // xml data type and struct
    var types = new Array(
        "ParameterValueStruct",
        /*
        [
            {
                Name: string,
                Value: string,
                ValueType: string,
                ReadWrite: bool
            },
            ...
        ]
        */
        
        "Page",
        /*
        [
            {
                DisplayName: string,
                Params: [
                    {
                        DisplayName: string,
                        Name: string,
                        ViewType: string,
                        ValueType: string,
                        Value: [[Keys(string)], [Values(string)]],
                        ReadOnly: bool
                    },
                    ...
                ]
            },
            ...
        ]
        */
        
        "CpeInfo",
        /*
        [
            {
                Mac: string,
                CpeAddress: string
            },
            ...
        ]
        */
        
        "EgsDoorStatus"
        /*
        [
            {
                Mac: string,
                DoorStatus: bool
            },
            ...
        ]
        */
      );
    
    var getNodeVal = function (node) {
            if (node != undefined && node.firstChild != null) {
                return node.firstChild.nodeValue;
            }
            return "";
        },
        getBoolVal = function (node) {
            return getNodeVal(node) == "true" ? true : false;
        },
        getAttribute = function (node, attrName) {
            if (node != undefined && node.attributes.length != 0) {
                return node.getAttribute(attrName);
            }
            return "";
        };
    
    var getParamValStructs = function (nodeList) {
            if (nodeList == null || nodeList.length == 0) return null;
            
            var result = new Array();
            
            for (var i = 0; i < nodeList.length; i++) {
                var node = nodeList[i];
                var paramVal = {
                    Name:       getNodeVal(node.getElementsByTagName("Name")[0]),
                    Value:      getNodeVal(node.getElementsByTagName("Value")[0]),
                    ValueType:  getNodeVal(node.getElementsByTagName("ValueType")[0]),
                    ReadWrite:  getBoolVal(node.getElementsByTagName("ReadWrite")[0])
                };

                result[result.length] = paramVal;
            }
            return result;
        },
        getPages = function (nodeList) {
            if (nodeList == null || nodeList.length == 0) return null;
            
            var result = new Array();
            var getVals = function (node) {
                if (node != undefined && node.firstChild != null) {
                    var result = new Array(), keys = new Array(), values = new Array();
                    
                    if (node.childNodes.length > 1) {
                        var items = node.getElementsByTagName("Item");
                        for (var i=0; i<items.length; i++) {
                            keys[keys.length] = getAttribute(items[i], "displayname");
                            values[values.length] = getNodeVal(items[i]);
                        }
                    } else {
                        keys[keys.length] = "Value";
                        values[values.length] = getNodeVal(node);
                    }
                    result[result.length] = keys;
                    result[result.length] = values;
                    return result;
                }
                return null;
            };
            
            for (var i=0; i<nodeList.length; i++) {
                var params = new Array(), paramNodeList = nodeList[i].getElementsByTagName("Param");
                
                for (var j=0; j<paramNodeList.length; j++) {
                    var param = {
                        DisplayName:    getAttribute(paramNodeList[j], "displayname"),
                        Name:           getNodeVal(paramNodeList[j].getElementsByTagName("Name")[0]),
                        ViewType:       getNodeVal(paramNodeList[j].getElementsByTagName("ViewType")[0]),
                        ValueType:      getNodeVal(paramNodeList[j].getElementsByTagName("ValueType")[0]),
                        Value:          getVals(paramNodeList[j].getElementsByTagName("Value")[0]),
                        ReadOnly:       getBoolVal(paramNodeList[j].getElementsByTagName("ReadOnly")[0])
                    };
                    params[params.length] = param;
                }
                
                var page = {
                    DisplayName: getAttribute(nodeList[i], "displayname"),
                    Params: params
                };
                result[result.length] = page;
            }
            return result;
        },
        getCpeInfos = function (nodeList) {
            if (nodeList == null || nodeList.length == 0) return null;
            
            var result = new Array();
            
            for (var i = 0; i < nodeList.length; i++) {
                var node = nodeList[i];
                var info = {
                    Mac:        getNodeVal(node.getElementsByTagName("Mac")[0]),
                    CpeAddress: getNodeVal(node.getElementsByTagName("CpeAddress")[0])
                };

                result[result.length] = info;
            }
            return result;
        },
        getEgsDoorStatus = function (nodeList) {
            if (nodeList == null || nodeList.length == 0) return null;
            
            var result = new Array();
            
            for (var i = 0; i < nodeList.length; i++) {
                var node = nodeList[i];
                var status = {
                    Show:       getNodeVal(node.getElementsByTagName("Show")[0]),
                    DoorStatus: getBoolVal(node.getElementsByTagName("DoorStatus")[0])
                };

                result[result.length] = status;
            }
            return result;
        };
    
    self.parse = function (xmlDoc) {
        try {
            for (var i = 0; i < types.length; i++) {
                if (xmlDoc.childNodes.item(0).childNodes.item(1).nodeName.indexOf(types[i]) >= 0) {
                    var nodeList = xmlDoc.getElementsByTagName(types[i]);
                    switch (types[i]) {
                        case "ParameterValueStruct":    return getParamValStructs(nodeList);
                        case "Page":                    return getPages(nodeList);
                        case "CpeInfo":                 return getCpeInfos(nodeList);
                        case "EgsDoorStatus":           return getEgsDoorStatus(nodeList);
                    }
                }
            }
        } catch (e) {
            console.log("parse error: " + e.message);
            return null;   
        }
    };
};